import { StorageHandler, EventBus as EventBus$1, ReadOnlyStorageHandler } from 'live-connect-common';

type StorageStrategy = 'cookie' | 'ls' | 'none' | 'disabled';

type Wrapped<T> = NonNullable<T> | (() => undefined);
declare class WrappingContext<T extends object> {
    private obj;
    private name;
    private errors;
    private eventBus;
    constructor(obj: T, name: string, eventBus: EventBus);
    wrap<K extends keyof T & string>(functionName: K): Wrapped<T[K]>;
    reportErrors(): void;
}

declare class WrappedReadOnlyStorageHandler implements ReadOnlyStorageHandler {
    private minimalFunctions;
    protected constructor(storageStrategy: StorageStrategy, wrapper: WrappingContext<ReadOnlyStorageHandler>);
    static make(storageStrategy: StorageStrategy, externalStorageHandler: ReadOnlyStorageHandler, eventBus: EventBus$1): WrappedReadOnlyStorageHandler;
    localStorageIsEnabled(): boolean;
    getCookie(key: string): string | null;
    getDataFromLocalStorage(key: string): string | null;
}
declare class WrappedStorageHandler extends WrappedReadOnlyStorageHandler implements StorageHandler {
    storageStrategy: StorageStrategy;
    private functions;
    protected constructor(storageStrategy: StorageStrategy, wrapper: WrappingContext<StorageHandler>);
    static make(storageStrategy: StorageStrategy, externalStorageHandler: StorageHandler, eventBus: EventBus$1): WrappedStorageHandler;
    setCookie(key: string, value: string, expires?: Date, sameSite?: string, domain?: string): void;
    setDataInLocalStorage(key: string, value: string): void;
    removeDataFromLocalStorage(key: string): void;
    findSimilarCookies(substring: string): string[];
}

type RecordMetadata = {
    expiresAt?: Date;
    writtenAt: Date;
};
type CacheRecord = {
    data: string;
    meta: RecordMetadata;
};
interface DurableCache {
    get: (key: string) => CacheRecord | null;
    set: (key: string, value: string, expiration?: Date) => void;
}
type StorageHandlerBackedCacheArgs = {
    storageHandler: WrappedStorageHandler;
    eventBus: EventBus$1;
    cookieDomain: string;
};
declare class StorageHandlerBackedCache implements DurableCache {
    private handler;
    private cookieDomain;
    private eventBus;
    constructor(opts: StorageHandlerBackedCacheArgs);
    private deleteCookie;
    private parseMetaRecord;
    private serializeMetaRecord;
    private getCookieRecord;
    private getLSRecord;
    get(key: string): CacheRecord | null;
    set(key: string, value: string, expires?: Date): void;
}

type ResolutionMetadata = {
    expiresAt?: Date;
};

type UrlCollectionMode = 'full' | 'no_path';

interface IdentityResolutionConfig {
    url?: string;
    ajaxTimeout?: number;
    source?: string;
    publisherId?: number;
    requestedAttributes?: string[];
}
interface ProvidedIdCookieConfig {
    mode: 'provided';
    strategy: 'cookie' | 'localStorage';
    name: string;
}
interface GeneratedIdCookieConfig {
    mode: 'generated';
}
type IdCookieConfig = ProvidedIdCookieConfig | GeneratedIdCookieConfig;
interface LiveConnectConfig {
    appId?: string;
    wrapperName?: string;
    storageStrategy?: StorageStrategy;
    collectorUrl?: string;
    usPrivacyString?: string;
    gdprApplies?: boolean;
    gdprConsent?: string;
    expirationDays?: number;
    identifiersToResolve?: string | string[];
    trackerVersion?: string;
    identityResolutionConfig?: IdentityResolutionConfig;
    distributorId?: string;
    globalVarName?: string;
    urlCollectionMode?: UrlCollectionMode;
    queryParametersFilter?: string;
    ajaxTimeout?: number;
    contextSelectors?: string;
    contextElementsLength?: number;
    peopleVerifiedId?: string;
    gppString?: string;
    gppApplicableSections?: number[];
    idCookie?: IdCookieConfig;
}
type ResolutionParams = Record<string, string | string[]>;
type IdentityResultionResult = unknown;
interface ErrorBus {
    emitErrorWithMessage(name: string, message: string, e?: unknown): this;
    emitError(name: string, exception?: unknown): this;
}
interface EventBus extends ErrorBus {
    on<F extends ((event: unknown) => void)>(name: string, callback: F, ctx?: ThisParameterType<F>): this;
    once<F extends ((event: unknown) => void)>(name: string, callback: F, ctx?: ThisParameterType<F>): this;
    emit(name: string, event: unknown): this;
    off(name: string, callback: (event: unknown) => void): this;
}
interface PublicLiveConnect {
    push?: (...event: unknown[]) => void;
    resolve?: (successCallBack: (result: IdentityResultionResult, meta: ResolutionMetadata) => void, errorCallBack: () => void, additionalParams?: ResolutionParams) => void;
}
interface InternalLiveConnect extends PublicLiveConnect {
    ready?: boolean;
    fire?: () => void;
    resolutionCallUrl?: (additionalParams: ResolutionParams) => string;
    peopleVerifiedId?: string;
    config?: LiveConnectConfig;
    eventBus?: EventBus;
    storageHandler?: WrappedStorageHandler;
    cache?: DurableCache;
}

export { type DurableCache as D, type EventBus as E, type InternalLiveConnect as I, type LiveConnectConfig as L, type PublicLiveConnect as P, StorageHandlerBackedCache as S, WrappedStorageHandler as W };
